@@ -76,12 +76,11 @@ struct _GdkWindowImplWin32
   GdkWin32PositionInfo position_info;
 
   HCURSOR hcursor;
+  HICON   hicon;
 
   /* Window size hints */
   gint hint_flags;
-  gint hint_x, hint_y;
-  gint hint_min_width, hint_min_height;
-  gint hint_max_width, hint_max_height;
+  GdkGeometry hints;
 
   gboolean extension_events_selected;
 };
