@@ -109,10 +109,8 @@ gdk_gc_x11_finalize (GObject *object)
   if (x11_gc->clip_region)
     gdk_region_destroy (x11_gc->clip_region);
   
-#if HAVE_XFT
   if (x11_gc->fg_picture != None)
     XRenderFreePicture (GDK_GC_XDISPLAY (x11_gc), x11_gc->fg_picture);
-#endif  
   
   XFreeGC (GDK_GC_XDISPLAY (x11_gc), GDK_GC_XGC (x11_gc));
 
@@ -140,6 +138,7 @@ _gdk_x11_gc_new (GdkDrawable      *drawable,
   private = GDK_GC_X11 (gc);
 
   private->dirty_mask = 0;
+  private->have_clip_mask = FALSE;
   private->clip_region = NULL;
     
   private->screen = GDK_DRAWABLE_IMPL_X11 (drawable)->screen;
@@ -158,7 +157,10 @@ _gdk_x11_gc_new (GdkDrawable      *drawable,
 
   if (values_mask & GDK_GC_FOREGROUND)
     private->fg_pixel = values->foreground.pixel;
-  
+
+  if ((values_mask & GDK_GC_CLIP_MASK) && values->clip_mask)
+    private->have_clip_mask = TRUE;
+
   xvalues.function = GXcopy;
   xvalues.fill_style = FillSolid;
   xvalues.arc_mode = ArcPieSlice;
@@ -373,8 +375,6 @@ gdk_x11_gc_set_values (GdkGC           *gc,
   XGCValues xvalues;
   unsigned long xvalues_mask = 0;
 
-  g_return_if_fail (GDK_IS_GC (gc));
-
   x11_gc = GDK_GC_X11 (gc);
 
   if (values_mask & (GDK_GC_CLIP_X_ORIGIN | GDK_GC_CLIP_Y_ORIGIN))
@@ -396,6 +396,8 @@ gdk_x11_gc_set_values (GdkGC           *gc,
 	  gdk_region_destroy (x11_gc->clip_region);
 	  x11_gc->clip_region = NULL;
 	}
+
+      x11_gc->have_clip_mask = values->clip_mask != NULL;
     }
 
   if (values_mask & GDK_GC_FOREGROUND)
@@ -645,20 +647,30 @@ gdk_gc_set_clip_rectangle (GdkGC	*gc,
 			   GdkRectangle *rectangle)
 {
   GdkGCX11 *x11_gc;
-
+  gboolean had_region = FALSE;
+  
   g_return_if_fail (GDK_IS_GC (gc));
 
   x11_gc = GDK_GC_X11 (gc);
 
   if (x11_gc->clip_region)
-    gdk_region_destroy (x11_gc->clip_region);
+    {
+      had_region = TRUE;
+      gdk_region_destroy (x11_gc->clip_region);
+    }
 
   if (rectangle)
     x11_gc->clip_region = gdk_region_rectangle (rectangle);
   else
+    x11_gc->clip_region = NULL;
+
+  /* Unset immediately, to make sure Xlib doesn't keep the
+   * XID of an old clip mask cached
+   */
+  if ((had_region && !rectangle) || x11_gc->have_clip_mask)
     {
-      x11_gc->clip_region = NULL;
       XSetClipMask (GDK_GC_XDISPLAY (gc), GDK_GC_XGC (gc), None);
+      x11_gc->have_clip_mask = FALSE;
     }
 
   gc->clip_x_origin = 0;
@@ -672,22 +684,32 @@ gdk_gc_set_clip_region (GdkGC	  *gc,
 			GdkRegion *region)
 {
   GdkGCX11 *x11_gc;
+  gboolean had_region = FALSE;
 
   g_return_if_fail (GDK_IS_GC (gc));
 
   x11_gc = GDK_GC_X11 (gc);
 
   if (x11_gc->clip_region)
-    gdk_region_destroy (x11_gc->clip_region);
+    {
+      had_region = TRUE;
+      gdk_region_destroy (x11_gc->clip_region);
+    }
 
   if (region)
     x11_gc->clip_region = gdk_region_copy (region);
   else
+    x11_gc->clip_region = NULL;    
+
+  /* Unset immediately, to make sure Xlib doesn't keep the
+   * XID of an old clip mask cached
+   */
+  if ((had_region && !region) || x11_gc->have_clip_mask)
     {
-      x11_gc->clip_region = NULL;
       XSetClipMask (GDK_GC_XDISPLAY (gc), GDK_GC_XGC (gc), None);
+      x11_gc->have_clip_mask = FALSE;
     }
-  
+
   gc->clip_x_origin = 0;
   gc->clip_y_origin = 0;
   
@@ -792,7 +814,6 @@ gdk_x11_gc_get_xgc (GdkGC *gc)
   return gc_x11->xgc;
 }
 
-#ifdef HAVE_XFT
 /* Various bits of the below are roughly cribbed from XFree86
  * lib/Xft/xftdraw.c, Copyright 2000, Keith Packard
  */
@@ -918,5 +939,3 @@ _gdk_gc_x11_get_fg_xft_color (GdkGC    *gc,
   xftcolor->color.blue = color.blue;
   xftcolor->color.alpha = 0xffff;
 }
-
-#endif /* HAVE_XFT */
