@@ -6,7 +6,7 @@
 
 G_BEGIN_DECLS
 
-/* BOOLEAN:BOXED (gtkmarshalers.list:24) */
+/* BOOLEAN:BOXED (./gtkmarshalers.list:24) */
 extern void _gtk_marshal_BOOLEAN__BOXED (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -14,7 +14,7 @@ extern void _gtk_marshal_BOOLEAN__BOXED (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* BOOLEAN:BOXED,BOXED (gtkmarshalers.list:25) */
+/* BOOLEAN:BOXED,BOXED (./gtkmarshalers.list:25) */
 extern void _gtk_marshal_BOOLEAN__BOXED_BOXED (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -22,7 +22,7 @@ extern void _gtk_marshal_BOOLEAN__BOXED_BOXED (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* BOOLEAN:ENUM (gtkmarshalers.list:26) */
+/* BOOLEAN:ENUM (./gtkmarshalers.list:26) */
 extern void _gtk_marshal_BOOLEAN__ENUM (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -30,7 +30,7 @@ extern void _gtk_marshal_BOOLEAN__ENUM (GClosure     *closure,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
 
-/* BOOLEAN:ENUM,INT (gtkmarshalers.list:27) */
+/* BOOLEAN:ENUM,INT (./gtkmarshalers.list:27) */
 extern void _gtk_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -38,7 +38,7 @@ extern void _gtk_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* BOOLEAN:OBJECT,UINT,FLAGS (gtkmarshalers.list:28) */
+/* BOOLEAN:OBJECT,UINT,FLAGS (./gtkmarshalers.list:28) */
 extern void _gtk_marshal_BOOLEAN__OBJECT_UINT_FLAGS (GClosure     *closure,
                                                      GValue       *return_value,
                                                      guint         n_param_values,
@@ -46,7 +46,7 @@ extern void _gtk_marshal_BOOLEAN__OBJECT_UINT_FLAGS (GClosure     *closure,
                                                      gpointer      invocation_hint,
                                                      gpointer      marshal_data);
 
-/* BOOLEAN:OBJECT,INT,INT,UINT (gtkmarshalers.list:29) */
+/* BOOLEAN:OBJECT,INT,INT,UINT (./gtkmarshalers.list:29) */
 extern void _gtk_marshal_BOOLEAN__OBJECT_INT_INT_UINT (GClosure     *closure,
                                                        GValue       *return_value,
                                                        guint         n_param_values,
@@ -54,7 +54,7 @@ extern void _gtk_marshal_BOOLEAN__OBJECT_INT_INT_UINT (GClosure     *closure,
                                                        gpointer      invocation_hint,
                                                        gpointer      marshal_data);
 
-/* BOOLEAN:OBJECT,STRING,STRING,BOXED (gtkmarshalers.list:30) */
+/* BOOLEAN:OBJECT,STRING,STRING,BOXED (./gtkmarshalers.list:30) */
 extern void _gtk_marshal_BOOLEAN__OBJECT_STRING_STRING_BOXED (GClosure     *closure,
                                                               GValue       *return_value,
                                                               guint         n_param_values,
@@ -62,7 +62,15 @@ extern void _gtk_marshal_BOOLEAN__OBJECT_STRING_STRING_BOXED (GClosure     *clos
                                                               gpointer      invocation_hint,
                                                               gpointer      marshal_data);
 
-/* BOOLEAN:OBJECT,BOXED,BOXED (gtkmarshalers.list:31) */
+/* BOOLEAN:OBJECT,BOXED (./gtkmarshalers.list:31) */
+extern void _gtk_marshal_BOOLEAN__OBJECT_BOXED (GClosure     *closure,
+                                                GValue       *return_value,
+                                                guint         n_param_values,
+                                                const GValue *param_values,
+                                                gpointer      invocation_hint,
+                                                gpointer      marshal_data);
+
+/* BOOLEAN:OBJECT,BOXED,BOXED (./gtkmarshalers.list:32) */
 extern void _gtk_marshal_BOOLEAN__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
@@ -70,7 +78,15 @@ extern void _gtk_marshal_BOOLEAN__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                       gpointer      invocation_hint,
                                                       gpointer      marshal_data);
 
-/* BOOLEAN:INT,INT (gtkmarshalers.list:32) */
+/* BOOLEAN:OBJECT,STRING,STRING (./gtkmarshalers.list:33) */
+extern void _gtk_marshal_BOOLEAN__OBJECT_STRING_STRING (GClosure     *closure,
+                                                        GValue       *return_value,
+                                                        guint         n_param_values,
+                                                        const GValue *param_values,
+                                                        gpointer      invocation_hint,
+                                                        gpointer      marshal_data);
+
+/* BOOLEAN:INT,INT (./gtkmarshalers.list:34) */
 extern void _gtk_marshal_BOOLEAN__INT_INT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -78,7 +94,15 @@ extern void _gtk_marshal_BOOLEAN__INT_INT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* BOOLEAN:VOID (gtkmarshalers.list:33) */
+/* BOOLEAN:INT,INT,INT (./gtkmarshalers.list:35) */
+extern void _gtk_marshal_BOOLEAN__INT_INT_INT (GClosure     *closure,
+                                               GValue       *return_value,
+                                               guint         n_param_values,
+                                               const GValue *param_values,
+                                               gpointer      invocation_hint,
+                                               gpointer      marshal_data);
+
+/* BOOLEAN:VOID (./gtkmarshalers.list:36) */
 extern void _gtk_marshal_BOOLEAN__VOID (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -86,7 +110,7 @@ extern void _gtk_marshal_BOOLEAN__VOID (GClosure     *closure,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
 
-/* BOOLEAN:BOOLEAN (gtkmarshalers.list:34) */
+/* BOOLEAN:BOOLEAN (./gtkmarshalers.list:37) */
 extern void _gtk_marshal_BOOLEAN__BOOLEAN (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -94,10 +118,10 @@ extern void _gtk_marshal_BOOLEAN__BOOLEAN (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* BOOLEAN:NONE (gtkmarshalers.list:35) */
+/* BOOLEAN:NONE (./gtkmarshalers.list:38) */
 #define _gtk_marshal_BOOLEAN__NONE	_gtk_marshal_BOOLEAN__VOID
 
-/* BOOLEAN:BOOLEAN,BOOLEAN,BOOLEAN (gtkmarshalers.list:36) */
+/* BOOLEAN:BOOLEAN,BOOLEAN,BOOLEAN (./gtkmarshalers.list:39) */
 extern void _gtk_marshal_BOOLEAN__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                            GValue       *return_value,
                                                            guint         n_param_values,
@@ -105,7 +129,7 @@ extern void _gtk_marshal_BOOLEAN__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure
                                                            gpointer      invocation_hint,
                                                            gpointer      marshal_data);
 
-/* ENUM:ENUM (gtkmarshalers.list:37) */
+/* ENUM:ENUM (./gtkmarshalers.list:40) */
 extern void _gtk_marshal_ENUM__ENUM (GClosure     *closure,
                                      GValue       *return_value,
                                      guint         n_param_values,
@@ -113,7 +137,7 @@ extern void _gtk_marshal_ENUM__ENUM (GClosure     *closure,
                                      gpointer      invocation_hint,
                                      gpointer      marshal_data);
 
-/* INT:POINTER (gtkmarshalers.list:38) */
+/* INT:POINTER (./gtkmarshalers.list:41) */
 extern void _gtk_marshal_INT__POINTER (GClosure     *closure,
                                        GValue       *return_value,
                                        guint         n_param_values,
@@ -121,19 +145,19 @@ extern void _gtk_marshal_INT__POINTER (GClosure     *closure,
                                        gpointer      invocation_hint,
                                        gpointer      marshal_data);
 
-/* NONE:BOOLEAN (gtkmarshalers.list:39) */
+/* NONE:BOOLEAN (./gtkmarshalers.list:42) */
 #define _gtk_marshal_VOID__BOOLEAN	g_cclosure_marshal_VOID__BOOLEAN
 #define _gtk_marshal_NONE__BOOLEAN	_gtk_marshal_VOID__BOOLEAN
 
-/* NONE:ENUM (gtkmarshalers.list:40) */
+/* NONE:ENUM (./gtkmarshalers.list:43) */
 #define _gtk_marshal_VOID__ENUM	g_cclosure_marshal_VOID__ENUM
 #define _gtk_marshal_NONE__ENUM	_gtk_marshal_VOID__ENUM
 
-/* NONE:INT (gtkmarshalers.list:41) */
+/* NONE:INT (./gtkmarshalers.list:44) */
 #define _gtk_marshal_VOID__INT	g_cclosure_marshal_VOID__INT
 #define _gtk_marshal_NONE__INT	_gtk_marshal_VOID__INT
 
-/* NONE:INT,BOOL (gtkmarshalers.list:42) */
+/* NONE:INT,BOOL (./gtkmarshalers.list:45) */
 extern void _gtk_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -142,7 +166,7 @@ extern void _gtk_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
                                             gpointer      marshal_data);
 #define _gtk_marshal_NONE__INT_BOOL	_gtk_marshal_VOID__INT_BOOLEAN
 
-/* NONE:INT,INT (gtkmarshalers.list:43) */
+/* NONE:INT,INT (./gtkmarshalers.list:46) */
 extern void _gtk_marshal_VOID__INT_INT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -151,11 +175,11 @@ extern void _gtk_marshal_VOID__INT_INT (GClosure     *closure,
                                         gpointer      marshal_data);
 #define _gtk_marshal_NONE__INT_INT	_gtk_marshal_VOID__INT_INT
 
-/* NONE:NONE (gtkmarshalers.list:44) */
+/* NONE:NONE (./gtkmarshalers.list:47) */
 #define _gtk_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID
 #define _gtk_marshal_NONE__NONE	_gtk_marshal_VOID__VOID
 
-/* NONE:STRING,INT,POINTER (gtkmarshalers.list:45) */
+/* NONE:STRING,INT,POINTER (./gtkmarshalers.list:48) */
 extern void _gtk_marshal_VOID__STRING_INT_POINTER (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -164,7 +188,7 @@ extern void _gtk_marshal_VOID__STRING_INT_POINTER (GClosure     *closure,
                                                    gpointer      marshal_data);
 #define _gtk_marshal_NONE__STRING_INT_POINTER	_gtk_marshal_VOID__STRING_INT_POINTER
 
-/* STRING:DOUBLE (gtkmarshalers.list:46) */
+/* STRING:DOUBLE (./gtkmarshalers.list:49) */
 extern void _gtk_marshal_STRING__DOUBLE (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -172,12 +196,12 @@ extern void _gtk_marshal_STRING__DOUBLE (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:DOUBLE (gtkmarshalers.list:47) */
+/* VOID:DOUBLE (./gtkmarshalers.list:50) */
 #define _gtk_marshal_VOID__DOUBLE	g_cclosure_marshal_VOID__DOUBLE
 
-/* VOID:BOOLEAN (gtkmarshalers.list:48) */
+/* VOID:BOOLEAN (./gtkmarshalers.list:51) */
 
-/* VOID:BOOLEAN,BOOLEAN,BOOLEAN (gtkmarshalers.list:49) */
+/* VOID:BOOLEAN,BOOLEAN,BOOLEAN (./gtkmarshalers.list:52) */
 extern void _gtk_marshal_VOID__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -185,10 +209,10 @@ extern void _gtk_marshal_VOID__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:BOXED (gtkmarshalers.list:50) */
+/* VOID:BOXED (./gtkmarshalers.list:53) */
 #define _gtk_marshal_VOID__BOXED	g_cclosure_marshal_VOID__BOXED
 
-/* VOID:BOXED,BOXED (gtkmarshalers.list:51) */
+/* VOID:BOXED,BOXED (./gtkmarshalers.list:54) */
 extern void _gtk_marshal_VOID__BOXED_BOXED (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -196,7 +220,7 @@ extern void _gtk_marshal_VOID__BOXED_BOXED (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:BOXED,BOXED,POINTER (gtkmarshalers.list:52) */
+/* VOID:BOXED,BOXED,POINTER (./gtkmarshalers.list:55) */
 extern void _gtk_marshal_VOID__BOXED_BOXED_POINTER (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
@@ -204,7 +228,7 @@ extern void _gtk_marshal_VOID__BOXED_BOXED_POINTER (GClosure     *closure,
                                                     gpointer      invocation_hint,
                                                     gpointer      marshal_data);
 
-/* VOID:BOXED,OBJECT (gtkmarshalers.list:53) */
+/* VOID:BOXED,OBJECT (./gtkmarshalers.list:56) */
 extern void _gtk_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -212,7 +236,7 @@ extern void _gtk_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:BOXED,STRING,INT (gtkmarshalers.list:54) */
+/* VOID:BOXED,STRING,INT (./gtkmarshalers.list:57) */
 extern void _gtk_marshal_VOID__BOXED_STRING_INT (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -220,7 +244,7 @@ extern void _gtk_marshal_VOID__BOXED_STRING_INT (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:BOXED,UINT (gtkmarshalers.list:55) */
+/* VOID:BOXED,UINT (./gtkmarshalers.list:58) */
 extern void _gtk_marshal_VOID__BOXED_UINT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -228,7 +252,7 @@ extern void _gtk_marshal_VOID__BOXED_UINT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:BOXED,UINT,FLAGS (gtkmarshalers.list:56) */
+/* VOID:BOXED,UINT,FLAGS (./gtkmarshalers.list:59) */
 extern void _gtk_marshal_VOID__BOXED_UINT_FLAGS (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -236,7 +260,7 @@ extern void _gtk_marshal_VOID__BOXED_UINT_FLAGS (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:BOXED,UINT,UINT (gtkmarshalers.list:57) */
+/* VOID:BOXED,UINT,UINT (./gtkmarshalers.list:60) */
 extern void _gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -244,9 +268,9 @@ extern void _gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
-/* VOID:ENUM (gtkmarshalers.list:58) */
+/* VOID:ENUM (./gtkmarshalers.list:61) */
 
-/* VOID:ENUM,BOOLEAN (gtkmarshalers.list:59) */
+/* VOID:ENUM,BOOLEAN (./gtkmarshalers.list:62) */
 extern void _gtk_marshal_VOID__ENUM_BOOLEAN (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -254,7 +278,7 @@ extern void _gtk_marshal_VOID__ENUM_BOOLEAN (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:ENUM,ENUM (gtkmarshalers.list:60) */
+/* VOID:ENUM,ENUM (./gtkmarshalers.list:63) */
 extern void _gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -262,7 +286,7 @@ extern void _gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:ENUM,FLOAT (gtkmarshalers.list:61) */
+/* VOID:ENUM,FLOAT (./gtkmarshalers.list:64) */
 extern void _gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -270,7 +294,7 @@ extern void _gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:ENUM,FLOAT,BOOLEAN (gtkmarshalers.list:62) */
+/* VOID:ENUM,FLOAT,BOOLEAN (./gtkmarshalers.list:65) */
 extern void _gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -278,7 +302,7 @@ extern void _gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:ENUM,INT (gtkmarshalers.list:63) */
+/* VOID:ENUM,INT (./gtkmarshalers.list:66) */
 extern void _gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -286,7 +310,7 @@ extern void _gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:ENUM,INT,BOOLEAN (gtkmarshalers.list:64) */
+/* VOID:ENUM,INT,BOOLEAN (./gtkmarshalers.list:67) */
 extern void _gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -294,11 +318,11 @@ extern void _gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:INT (gtkmarshalers.list:65) */
+/* VOID:INT (./gtkmarshalers.list:68) */
 
-/* VOID:INT,INT (gtkmarshalers.list:66) */
+/* VOID:INT,INT (./gtkmarshalers.list:69) */
 
-/* VOID:INT,INT,BOXED (gtkmarshalers.list:67) */
+/* VOID:INT,INT,BOXED (./gtkmarshalers.list:70) */
 extern void _gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -306,7 +330,7 @@ extern void _gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:INT,INT,INT (gtkmarshalers.list:68) */
+/* VOID:INT,INT,INT (./gtkmarshalers.list:71) */
 extern void _gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -314,10 +338,10 @@ extern void _gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:OBJECT (gtkmarshalers.list:69) */
+/* VOID:OBJECT (./gtkmarshalers.list:72) */
 #define _gtk_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
 
-/* VOID:OBJECT,BOOLEAN (gtkmarshalers.list:70) */
+/* VOID:OBJECT,BOOLEAN (./gtkmarshalers.list:73) */
 extern void _gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -325,7 +349,7 @@ extern void _gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,BOXED,BOXED (gtkmarshalers.list:71) */
+/* VOID:OBJECT,BOXED,BOXED (./gtkmarshalers.list:74) */
 extern void _gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -333,7 +357,7 @@ extern void _gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshalers.list:72) */
+/* VOID:OBJECT,BOXED,UINT,UINT (./gtkmarshalers.list:75) */
 extern void _gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                                        GValue       *return_value,
                                                        guint         n_param_values,
@@ -341,7 +365,7 @@ extern void _gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                                        gpointer      invocation_hint,
                                                        gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT (gtkmarshalers.list:73) */
+/* VOID:OBJECT,INT,INT (./gtkmarshalers.list:76) */
 extern void _gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -349,7 +373,7 @@ extern void _gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshalers.list:74) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./gtkmarshalers.list:77) */
 extern void _gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                                GValue       *return_value,
                                                                guint         n_param_values,
@@ -357,7 +381,7 @@ extern void _gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *clo
                                                                gpointer      invocation_hint,
                                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,OBJECT (gtkmarshalers.list:75) */
+/* VOID:OBJECT,OBJECT (./gtkmarshalers.list:78) */
 extern void _gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -365,7 +389,7 @@ extern void _gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:OBJECT,STRING,STRING (gtkmarshalers.list:76) */
+/* VOID:OBJECT,STRING,STRING (./gtkmarshalers.list:79) */
 extern void _gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                                      GValue       *return_value,
                                                      guint         n_param_values,
@@ -373,7 +397,7 @@ extern void _gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                                      gpointer      invocation_hint,
                                                      gpointer      marshal_data);
 
-/* VOID:OBJECT,UINT (gtkmarshalers.list:77) */
+/* VOID:OBJECT,UINT (./gtkmarshalers.list:80) */
 extern void _gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -381,7 +405,7 @@ extern void _gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:OBJECT,UINT,FLAGS (gtkmarshalers.list:78) */
+/* VOID:OBJECT,UINT,FLAGS (./gtkmarshalers.list:81) */
 extern void _gtk_marshal_VOID__OBJECT_UINT_FLAGS (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -389,10 +413,10 @@ extern void _gtk_marshal_VOID__OBJECT_UINT_FLAGS (GClosure     *closure,
                                                   gpointer      invocation_hint,
                                                   gpointer      marshal_data);
 
-/* VOID:POINTER (gtkmarshalers.list:79) */
+/* VOID:POINTER (./gtkmarshalers.list:82) */
 #define _gtk_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER
 
-/* VOID:POINTER,INT (gtkmarshalers.list:80) */
+/* VOID:POINTER,INT (./gtkmarshalers.list:83) */
 extern void _gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -400,7 +424,7 @@ extern void _gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER,POINTER (gtkmarshalers.list:81) */
+/* VOID:POINTER,POINTER,POINTER (./gtkmarshalers.list:84) */
 extern void _gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -408,7 +432,7 @@ extern void _gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:POINTER,UINT (gtkmarshalers.list:82) */
+/* VOID:POINTER,UINT (./gtkmarshalers.list:85) */
 extern void _gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -416,10 +440,10 @@ extern void _gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING (gtkmarshalers.list:83) */
+/* VOID:STRING (./gtkmarshalers.list:86) */
 #define _gtk_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
 
-/* VOID:STRING,STRING (gtkmarshalers.list:84) */
+/* VOID:STRING,STRING (./gtkmarshalers.list:87) */
 extern void _gtk_marshal_VOID__STRING_STRING (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -427,9 +451,9 @@ extern void _gtk_marshal_VOID__STRING_STRING (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:STRING,INT,POINTER (gtkmarshalers.list:85) */
+/* VOID:STRING,INT,POINTER (./gtkmarshalers.list:88) */
 
-/* VOID:UINT,FLAGS,BOXED (gtkmarshalers.list:86) */
+/* VOID:UINT,FLAGS,BOXED (./gtkmarshalers.list:89) */
 extern void _gtk_marshal_VOID__UINT_FLAGS_BOXED (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -437,7 +461,7 @@ extern void _gtk_marshal_VOID__UINT_FLAGS_BOXED (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:UINT,UINT (gtkmarshalers.list:87) */
+/* VOID:UINT,UINT (./gtkmarshalers.list:90) */
 extern void _gtk_marshal_VOID__UINT_UINT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -445,7 +469,7 @@ extern void _gtk_marshal_VOID__UINT_UINT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:UINT,STRING (gtkmarshalers.list:88) */
+/* VOID:UINT,STRING (./gtkmarshalers.list:91) */
 extern void _gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -453,7 +477,7 @@ extern void _gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshalers.list:89) */
+/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (./gtkmarshalers.list:92) */
 extern void _gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
@@ -461,7 +485,7 @@ extern void _gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closur
                                                             gpointer      invocation_hint,
                                                             gpointer      marshal_data);
 
-/* VOID:UINT,OBJECT,UINT,FLAGS,FLAGS (gtkmarshalers.list:90) */
+/* VOID:UINT,OBJECT,UINT,FLAGS,FLAGS (./gtkmarshalers.list:93) */
 extern void _gtk_marshal_VOID__UINT_OBJECT_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -469,7 +493,7 @@ extern void _gtk_marshal_VOID__UINT_OBJECT_UINT_FLAGS_FLAGS (GClosure     *closu
                                                              gpointer      invocation_hint,
                                                              gpointer      marshal_data);
 
-/* VOID:VOID (gtkmarshalers.list:91) */
+/* VOID:VOID (./gtkmarshalers.list:94) */
 
 G_END_DECLS
 
