@@ -35,12 +35,8 @@
 #include "gtkmenuitem.h"
 #include "gtkseparatormenuitem.h"
 
-
-#define BORDER_SPACING  3
-
 #define MENU_ITEM_CLASS(w)  GTK_MENU_ITEM_CLASS (GTK_OBJECT (w)->klass)
 
-
 enum {
   ACTIVATE,
   ACTIVATE_ITEM,
@@ -211,6 +207,33 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
 								     GTK_SHADOW_NONE,
 								     G_PARAM_READABLE),
 						  gtk_rc_property_parse_enum);
+
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_int ("horizontal_padding",
+							     "Horizontal Padding",
+							     "Padding to left and right of the menu item",
+							     0,
+							     G_MAXINT,
+							     3,
+							     G_PARAM_READABLE));
+
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_int ("toggle_spacing",
+							     "Icon Spacing",
+							     "Space between icon and label",
+							     0,
+							     G_MAXINT,
+							     5,
+							     G_PARAM_READABLE));
+
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_int ("arrow_spacing",
+							     "Arrow Spacing",
+							     "Space between label and arrow",
+							     0,
+							     G_MAXINT,
+							     10,
+							     G_PARAM_READABLE));
 }
 
 static void
@@ -449,16 +472,20 @@ gtk_menu_item_size_request (GtkWidget      *widget,
   GtkMenuItem *menu_item;
   GtkBin *bin;
   guint accel_width;
+  guint horizontal_padding;
 
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
   g_return_if_fail (requisition != NULL);
 
+  gtk_widget_style_get (widget,
+ 			"horizontal padding", &horizontal_padding,
+			NULL);
+  
   bin = GTK_BIN (widget);
   menu_item = GTK_MENU_ITEM (widget);
 
   requisition->width = (GTK_CONTAINER (widget)->border_width +
-			widget->style->xthickness +
-			BORDER_SPACING) * 2;
+			widget->style->xthickness + horizontal_padding) * 2;
   requisition->height = (GTK_CONTAINER (widget)->border_width +
 			 widget->style->ythickness) * 2;
 
@@ -472,7 +499,21 @@ gtk_menu_item_size_request (GtkWidget      *widget,
       requisition->height += child_requisition.height;
 
       if (menu_item->submenu && menu_item->show_submenu_indicator)
-	requisition->width += child_requisition.height;
+	{
+	  guint arrow_spacing;
+	  
+	  gtk_widget_style_get (widget,
+				"arrow_spacing", &arrow_spacing,
+				NULL);
+
+	  requisition->width += child_requisition.height;
+	  requisition->width += arrow_spacing;
+	}
+    }
+  else
+    {
+      /* separator item */
+      requisition->height += 4;
     }
 
   accel_width = 0;
@@ -504,10 +545,15 @@ gtk_menu_item_size_allocate (GtkWidget     *widget,
   if (bin->child)
     {
       GtkRequisition child_requisition;
+      guint horizontal_padding;
+
+      gtk_widget_style_get (widget,
+			    "horizontal_padding", &horizontal_padding,
+			    NULL);
       
       child_allocation.x = (GTK_CONTAINER (widget)->border_width +
                             widget->style->xthickness +
-			    BORDER_SPACING);
+			    horizontal_padding);
       child_allocation.y = (GTK_CONTAINER (widget)->border_width +
 			    widget->style->ythickness);
       child_allocation.width = MAX (1, (gint)allocation->width - child_allocation.x * 2);
@@ -560,7 +606,6 @@ gtk_menu_item_realize (GtkWidget *widget)
   attributes.window_type = GDK_WINDOW_CHILD;
   attributes.wclass = GDK_INPUT_ONLY;
   attributes.event_mask = (gtk_widget_get_events (widget) |
-			   GDK_EXPOSURE_MASK |
 			   GDK_BUTTON_PRESS_MASK |
 			   GDK_BUTTON_RELEASE_MASK |
 			   GDK_ENTER_NOTIFY_MASK |
@@ -649,29 +694,36 @@ gtk_menu_item_paint (GtkWidget    *widget,
 	  gint arrow_x, arrow_y;
 	  gint arrow_size;
 	  gint arrow_extent;
+	  guint horizontal_padding;
 	  GtkTextDirection direction;
 	  GtkArrowType arrow_type;
 
 	  direction = gtk_widget_get_direction (widget);
       
+ 	  gtk_widget_style_get (widget,
+ 				"horizontal_padding", &horizontal_padding,
+ 				NULL);
+ 	  
 	  gtk_widget_get_child_requisition (GTK_BIN (menu_item)->child,
 					    &child_requisition);
 
 	  arrow_size = child_requisition.height - 2 * widget->style->ythickness;
-	  arrow_extent = arrow_size * 0.6;
+	  arrow_extent = arrow_size * 0.8;
 	  
 	  shadow_type = GTK_SHADOW_OUT;
 	  if (state_type == GTK_STATE_PRELIGHT)
 	    shadow_type = GTK_SHADOW_IN;
 
-	  if (direction == GTK_TEXT_DIR_LTR) {
-	    arrow_x = x + width - 1 - arrow_size + (arrow_size - arrow_extent) / 2;
-	    arrow_type = GTK_ARROW_RIGHT;
-	  }
-	  else {
-	    arrow_x = x + 1 + (arrow_size - arrow_extent) / 2;
-	    arrow_type = GTK_ARROW_LEFT;
-	  }
+	  if (direction == GTK_TEXT_DIR_LTR)
+	    {
+	      arrow_x = x + width - horizontal_padding - arrow_extent;
+	      arrow_type = GTK_ARROW_RIGHT;
+	    }
+	  else
+	    {
+	      arrow_x = x + horizontal_padding;
+	      arrow_type = GTK_ARROW_LEFT;
+	    }
 
 	  arrow_y = y + (height - arrow_extent) / 2;
 
@@ -684,10 +736,18 @@ gtk_menu_item_paint (GtkWidget    *widget,
 	}
       else if (!GTK_BIN (menu_item)->child)
 	{
+	  guint horizontal_padding;
+
+	  gtk_widget_style_get (widget,
+				"horizontal_padding", &horizontal_padding,
+				NULL);
+	  
 	  gtk_paint_hline (widget->style, widget->window, GTK_STATE_NORMAL,
 			   area, widget, "menuitem",
-			   widget->allocation.x, widget->allocation.width,
-			   widget->allocation.y);
+			   widget->allocation.x + horizontal_padding + widget->style->xthickness,
+			   widget->allocation.x + widget->allocation.width - horizontal_padding - widget->style->xthickness - 1,
+			   widget->allocation.y + (widget->allocation.height -
+						   widget->style->ythickness) / 2);
 	}
     }
 }
@@ -722,7 +782,7 @@ get_popup_delay (GtkMenuItem *menu_item)
     {
       gint popup_delay;
       
-      g_object_get (G_OBJECT (gtk_widget_get_settings (GTK_WIDGET (menu_item))),
+      g_object_get (gtk_widget_get_settings (GTK_WIDGET (menu_item)),
 		    "gtk-menu-popup-delay", &popup_delay,
 		    NULL);
 
@@ -744,7 +804,7 @@ gtk_real_menu_item_select (GtkItem *item)
       gint popup_delay;
 
       if (menu_item->timer)
-	gtk_timeout_remove (menu_item->timer);
+	g_source_remove (menu_item->timer);
 
       popup_delay = get_popup_delay (menu_item);
       
@@ -752,9 +812,9 @@ gtk_real_menu_item_select (GtkItem *item)
 	{
 	  GdkEvent *event = gtk_get_current_event ();
 	  
-	  menu_item->timer = gtk_timeout_add (popup_delay,
-					      gtk_menu_item_select_timeout,
-					      menu_item);
+	  menu_item->timer = g_timeout_add (popup_delay,
+					    gtk_menu_item_select_timeout,
+					    menu_item);
 	  if (event &&
 	      event->type != GDK_BUTTON_PRESS &&
 	      event->type != GDK_ENTER_NOTIFY)
@@ -786,7 +846,7 @@ gtk_real_menu_item_deselect (GtkItem *item)
     {
       if (menu_item->timer)
 	{
-	  gtk_timeout_remove (menu_item->timer);
+	  g_source_remove (menu_item->timer);
 	  menu_item->timer = 0;
 	}
       else
@@ -898,7 +958,7 @@ gtk_menu_item_popup_submenu (gpointer data)
   menu_item = GTK_MENU_ITEM (widget);
 
   if (menu_item->timer)
-    gtk_timeout_remove (menu_item->timer);
+    g_source_remove (menu_item->timer);
   menu_item->timer = 0;
 
   if (GTK_WIDGET_IS_SENSITIVE (menu_item->submenu))
@@ -911,6 +971,23 @@ gtk_menu_item_popup_submenu (gpointer data)
 		    0);
 }
 
+static void
+get_offsets (GtkMenu *menu,
+	     gint    *horizontal_offset,
+	     gint    *vertical_offset)
+{
+  gint vertical_padding;
+  
+  gtk_widget_style_get (GTK_WIDGET (menu),
+			"horizontal_offset", horizontal_offset,
+			"vertical_offset", vertical_offset,
+			"vertical_padding", &vertical_padding,
+			NULL);
+
+  *vertical_offset -= GTK_WIDGET (menu)->style->ythickness;
+  *vertical_offset -= vertical_padding;
+}
+
 static void
 gtk_menu_item_position_menu (GtkMenu  *menu,
 			     gint     *x,
@@ -927,6 +1004,9 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   GtkTextDirection direction;
   GdkRectangle monitor;
   gint monitor_num;
+  gint horizontal_offset;
+  gint vertical_offset;
+  gint parent_xthickness;
 
   g_return_if_fail (menu != NULL);
   g_return_if_fail (x != NULL);
@@ -935,6 +1015,9 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   menu_item = GTK_MENU_ITEM (user_data);
   widget = GTK_WIDGET (user_data);
 
+  if (push_in)
+    *push_in = FALSE;
+
   direction = gtk_widget_get_direction (widget);
 
   twidth = GTK_WIDGET (menu)->requisition.width;
@@ -955,6 +1038,8 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   tx += widget->allocation.x;
   ty += widget->allocation.y;
 
+  get_offsets (menu, &horizontal_offset, &vertical_offset);
+  
   switch (menu_item->submenu_placement)
     {
     case GTK_TOP_BOTTOM:
@@ -978,6 +1063,7 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
 
     case GTK_LEFT_RIGHT:
       parent_menu_item = GTK_MENU (widget->parent)->parent_menu_item;
+      parent_xthickness = widget->parent->style->xthickness;
       if (parent_menu_item && 
 	  !GTK_MENU (widget->parent)->torn_off && 
 	  !GTK_MENU_SHELL (menu)->active)
@@ -990,28 +1076,28 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
       switch (menu_item->submenu_direction)
 	{
 	case GTK_DIRECTION_LEFT:
-	  if ((tx - twidth) >= monitor.x)
-	    tx -= twidth;
+	  if ((tx - twidth - parent_xthickness - horizontal_offset) >= monitor.x)
+	    tx -= twidth + parent_xthickness + horizontal_offset;
 	  else
 	    {
 	      menu_item->submenu_direction = GTK_DIRECTION_RIGHT;
-	      tx += widget->allocation.width - 5;
+	      tx += widget->allocation.width + parent_xthickness + horizontal_offset;
 	    }
 	  break;
 
 	case GTK_DIRECTION_RIGHT:
-	  if ((tx + widget->allocation.width + twidth - 5) <= monitor.x + monitor.width)
-	    tx += widget->allocation.width - 5;
+	  if ((tx + widget->allocation.width + parent_xthickness + horizontal_offset + twidth) <= monitor.x + monitor.width)
+	    tx += widget->allocation.width + parent_xthickness + horizontal_offset;
 	  else
 	    {
 	      menu_item->submenu_direction = GTK_DIRECTION_LEFT;
-	      tx -= twidth;
+	      tx -= twidth + parent_xthickness + horizontal_offset;
 	    }
 	  break;
 	}
 
-      ty += widget->allocation.height / 4;
-
+      ty += vertical_offset;
+      
       /* If the height of the menu doesn't fit we move it upward. */
       ty = CLAMP (ty, monitor.y, MAX (monitor.y, monitor.y + monitor.height - theight));
       break;
