@@ -25,7 +25,6 @@
  */
 
 #include "gtkmenu.h"
-#include "gtksignal.h"
 #include "gtktearoffmenuitem.h"
 
 #define ARROW_SIZE 10
@@ -42,26 +41,29 @@ static void gtk_tearoff_menu_item_activate   (GtkMenuItem           *menu_item);
 static gint gtk_tearoff_menu_item_delete_cb  (GtkMenuItem           *menu_item,
 					      GdkEventAny           *event);
 
-GtkType
+GType
 gtk_tearoff_menu_item_get_type (void)
 {
-  static GtkType tearoff_menu_item_type = 0;
+  static GType tearoff_menu_item_type = 0;
 
   if (!tearoff_menu_item_type)
     {
-      static const GtkTypeInfo tearoff_menu_item_info =
+      static const GTypeInfo tearoff_menu_item_info =
       {
-        "GtkTearoffMenuItem",
-        sizeof (GtkTearoffMenuItem),
         sizeof (GtkTearoffMenuItemClass),
-        (GtkClassInitFunc) gtk_tearoff_menu_item_class_init,
-        (GtkObjectInitFunc) gtk_tearoff_menu_item_init,
-        /* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_tearoff_menu_item_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkTearoffMenuItem),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_tearoff_menu_item_init,
       };
 
-      tearoff_menu_item_type = gtk_type_unique (gtk_menu_item_get_type (), &tearoff_menu_item_info);
+      tearoff_menu_item_type =
+	g_type_register_static (GTK_TYPE_MENU_ITEM, "GtkTearoffMenuItem",
+			        &tearoff_menu_item_info, 0);
     }
 
   return tearoff_menu_item_type;
@@ -70,7 +72,7 @@ gtk_tearoff_menu_item_get_type (void)
 GtkWidget*
 gtk_tearoff_menu_item_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (gtk_tearoff_menu_item_get_type ()));
+  return g_object_new (GTK_TYPE_TEAROFF_MENU_ITEM, NULL);
 }
 
 static void
@@ -116,7 +118,7 @@ gtk_tearoff_menu_item_size_request (GtkWidget      *widget,
     }
   else
     {
-      requisition->height += widget->style->ythickness;
+      requisition->height += widget->style->ythickness + 4;
     }
 }
 
@@ -130,12 +132,16 @@ gtk_tearoff_menu_item_paint (GtkWidget   *widget,
   gint width, height;
   gint x, y;
   gint right_max;
-
+  GtkArrowType arrow_type;
+  GtkTextDirection direction;
+  
   if (GTK_WIDGET_DRAWABLE (widget))
     {
       menu_item = GTK_MENU_ITEM (widget);
       tearoff_item = GTK_TEAROFF_MENU_ITEM (widget);
 
+      direction = gtk_widget_get_direction (widget);
+
       x = widget->allocation.x + GTK_CONTAINER (menu_item)->border_width;
       y = widget->allocation.y + GTK_CONTAINER (menu_item)->border_width;
       width = widget->allocation.width - x * 2;
@@ -143,14 +149,21 @@ gtk_tearoff_menu_item_paint (GtkWidget   *widget,
       right_max = x + width;
 
       if (widget->state == GTK_STATE_PRELIGHT)
-	gtk_paint_box (widget->style,
-		       widget->window,
-		       GTK_STATE_PRELIGHT,
-		       GTK_SHADOW_OUT,
-		       area, widget, "menuitem",
-		       x, y, width, height);
-       else
-	 gdk_window_clear_area (widget->window, area->x, area->y, area->width, area->height);
+	{
+	  gint selected_shadow_type;
+	  
+	  gtk_widget_style_get (widget,
+				"selected_shadow_type", &selected_shadow_type,
+				NULL);
+	  gtk_paint_box (widget->style,
+			 widget->window,
+			 GTK_STATE_PRELIGHT,
+			 selected_shadow_type,
+			 area, widget, "menuitem",
+			 x, y, width, height);
+	}
+      else
+	gdk_window_clear_area (widget->window, area->x, area->y, area->width, area->height);
 
       if (tearoff_item->torn_off)
 	{
@@ -163,26 +176,54 @@ gtk_tearoff_menu_item_paint (GtkWidget   *widget,
 
 	  if (menu_item->toggle_size > ARROW_SIZE)
 	    {
-	      arrow_x = x + (menu_item->toggle_size - ARROW_SIZE)/2;
+	      if (direction == GTK_TEXT_DIR_LTR) {
+		arrow_x = x + (menu_item->toggle_size - ARROW_SIZE)/2;
+		arrow_type = GTK_ARROW_LEFT;
+	      }
+	      else {
+		arrow_x = x + width - menu_item->toggle_size + (menu_item->toggle_size - ARROW_SIZE)/2; 
+		arrow_type = GTK_ARROW_RIGHT;	    
+	      }
 	      x += menu_item->toggle_size + BORDER_SPACING;
 	    }
 	  else
 	    {
-	      arrow_x = ARROW_SIZE / 2;
+	      if (direction == GTK_TEXT_DIR_LTR) {
+		arrow_x = ARROW_SIZE / 2;
+		arrow_type = GTK_ARROW_LEFT;
+	      }
+	      else {
+		arrow_x = x + width - 2 * ARROW_SIZE + ARROW_SIZE / 2; 
+		arrow_type = GTK_ARROW_RIGHT;	    
+	      }
 	      x += 2 * ARROW_SIZE;
 	    }
-	  
-	  gtk_draw_arrow (widget->style, widget->window,
-			  widget->state, shadow_type, GTK_ARROW_LEFT, FALSE,
-			  arrow_x, y + height / 2 - 5, 
-			  ARROW_SIZE, ARROW_SIZE);
+
+
+	  gtk_paint_arrow (widget->style, widget->window,
+			   widget->state, shadow_type,
+			   NULL, widget, "tearoffmenuitem",
+			   arrow_type, FALSE,
+			   arrow_x, y + height / 2 - 5, 
+			   ARROW_SIZE, ARROW_SIZE);
 	}
 
       while (x < right_max)
 	{
-	  gtk_draw_hline (widget->style, widget->window, GTK_STATE_NORMAL,
-			  x, MIN (x+TEAR_LENGTH, right_max),
-			  y + (height - widget->style->ythickness)/2);
+	  gint x1, x2;
+
+	  if (direction == GTK_TEXT_DIR_LTR) {
+	    x1 = x;
+	    x2 = MIN (x + TEAR_LENGTH, right_max);
+	  }
+	  else {
+	    x1 = right_max - x;
+	    x2 = MAX (right_max - x - TEAR_LENGTH, 0);
+	  }
+	  
+	  gtk_paint_hline (widget->style, widget->window, GTK_STATE_NORMAL,
+			   NULL, widget, "tearoffmenuitem",
+			   x1, x2, y + (height - widget->style->ythickness) / 2);
 	  x += 2 * TEAR_LENGTH;
 	}
     }
@@ -223,10 +264,10 @@ gtk_tearoff_menu_item_activate (GtkMenuItem *menu_item)
 				    tearoff_menu_item->torn_off);
 
 	if (need_connect)
-	  gtk_signal_connect_object (GTK_OBJECT (menu->tearoff_window),  
-				     "delete_event",
-				     GTK_SIGNAL_FUNC (gtk_tearoff_menu_item_delete_cb),
-				     GTK_OBJECT (menu_item));
+	  g_signal_connect_swapped (menu->tearoff_window,
+				    "delete_event",
+				    G_CALLBACK (gtk_tearoff_menu_item_delete_cb),
+				    menu_item);
     }
 }
 
