@@ -104,6 +104,7 @@ struct _GtkTreeViewColumn
   guint maybe_reordered     : 1;
   guint reorderable         : 1;
   guint use_resized_width   : 1;
+  guint expand              : 1;
 };
 
 struct _GtkTreeViewColumnClass
@@ -177,6 +178,9 @@ void                    gtk_tree_view_column_clicked             (GtkTreeViewCol
 void                    gtk_tree_view_column_set_title           (GtkTreeViewColumn       *tree_column,
 								  const gchar             *title);
 G_CONST_RETURN gchar   *gtk_tree_view_column_get_title           (GtkTreeViewColumn       *tree_column);
+void                    gtk_tree_view_column_set_expand          (GtkTreeViewColumn       *tree_column,
+								  gboolean                 expand);
+gboolean                gtk_tree_view_column_get_expand          (GtkTreeViewColumn       *tree_column);
 void                    gtk_tree_view_column_set_clickable       (GtkTreeViewColumn       *tree_column,
 								  gboolean                 clickable);
 gboolean                gtk_tree_view_column_get_clickable       (GtkTreeViewColumn       *tree_column);
@@ -222,6 +226,11 @@ void                    gtk_tree_view_column_cell_get_size       (GtkTreeViewCol
 gboolean                gtk_tree_view_column_cell_is_visible     (GtkTreeViewColumn       *tree_column);
 void                    gtk_tree_view_column_focus_cell          (GtkTreeViewColumn       *tree_column,
 								  GtkCellRenderer         *cell);
+gboolean                gtk_tree_view_column_cell_get_position   (GtkTreeViewColumn       *tree_column,
+					                          GtkCellRenderer         *cell_renderer,
+					                          gint                    *start_pos,
+					                          gint                    *width);
+
 
 #ifdef __cplusplus
 }
